// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _URLPTRSplaySet_h
#ifdef __GNUG__
#pragma interface
#endif
#define _URLPTRSplaySet_h 1

#include "UP.Set.h"
#include "UP.SplayNode.h"

class URLPTRSplaySet : public URLPTRSet
{
protected:
  URLPTRSplayNode*   root;

  URLPTRSplayNode*   leftmost();
  URLPTRSplayNode*   rightmost();
  URLPTRSplayNode*   pred(URLPTRSplayNode* t);
  URLPTRSplayNode*   succ(URLPTRSplayNode* t);
  void            _kill(URLPTRSplayNode* t);
  URLPTRSplayNode*   _copy(URLPTRSplayNode* t);

public:
                  URLPTRSplaySet();
                  URLPTRSplaySet(URLPTRSplaySet& a);
                  ~URLPTRSplaySet();

  Pix             add(URLPTR  item);
  void            del(URLPTR  item);
  int             contains(URLPTR  item);

  void            clear();

  Pix             first();
  void            next(Pix& i);
  URLPTR&            operator () (Pix i);
  Pix             seek(URLPTR  item);

  Pix             last();
  void            prev(Pix& i);

  URLPTRSplaySet&    operator =  (const URLPTRSplaySet& b);
  void            operator |= (URLPTRSplaySet& b);
  void            operator -= (URLPTRSplaySet& b);
  void            operator &= (URLPTRSplaySet& b);

  int             operator == (URLPTRSplaySet& b);
  int             operator != (URLPTRSplaySet& b);
  int             operator <= (URLPTRSplaySet& b); 

  int             OK();
};


inline URLPTRSplaySet::~URLPTRSplaySet()
{
  _kill(root);
}

inline URLPTRSplaySet::URLPTRSplaySet()
{
  root = 0;
  count = 0;
}

inline URLPTRSplaySet::URLPTRSplaySet(URLPTRSplaySet& b)
{
  count = b.count;
  root = _copy(b.root);
}


inline URLPTRSplaySet& URLPTRSplaySet::operator = (const URLPTRSplaySet& b)
{
  if (this != &b)
    {
      _kill (root);
      count = b.count;
      root = _copy (b.root);
    }
  return *this;
}

inline int URLPTRSplaySet::operator != (URLPTRSplaySet& b)
{
  return ! (*this == b);
}

inline Pix URLPTRSplaySet::first()
{
  return Pix(leftmost());
}

inline Pix URLPTRSplaySet::last()
{
  return Pix(rightmost());
}

inline void URLPTRSplaySet::next(Pix& i)
{
  if (i != 0) i = Pix(succ((URLPTRSplayNode*)i));
}

inline void URLPTRSplaySet::prev(Pix& i)
{
  if (i != 0) i = Pix(pred((URLPTRSplayNode*)i));
}

inline URLPTR& URLPTRSplaySet::operator () (Pix i)
{
  if (i == 0) error("null Pix");
  return ((URLPTRSplayNode*)i)->item;
}

inline void URLPTRSplaySet::clear()
{
  _kill(root);
  count = 0;
  root = 0;
}

inline int URLPTRSplaySet::contains(URLPTR  key)
{
  return seek(key) != 0;
}

#endif
